using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace MapEditor
{
    public partial class MapEditor : Form
    {
        public MapEditor()
        {
            InitializeComponent();
        }
        
        //Event variables
        bool TilesLoaded = false;//have the tiles been loaded yet?
        bool MapLoaded = false;//has the map been loaded?

        Bitmap[] bmpTile;//array of tile images
        Bitmap bmpTileSet;//holds the set of tiles
        Graphics graphics;//for drawing stuff...
        int[,] Map; //2d array of tile ID's
        int SelectedTile;//int of tile ID of currently selected tile

        //store some basic integers so we don't need to get them from the textbox every time
        int TileWidth, TileHeight;//width and height of the tiles
        int TilesAcross, TilesDown;//how many tiles the tileset has
        int TileBorder;//how many pixels the border around each title has

        //map property variables
        int MapTileWidth, MapTileHeight;//how many tiles across and down the map has

        //mouse?
        bool isMouseDown = false;//this allows us to drag draw using the timer instead of having to click
        //in the picturebox

        //loads the tileset and tiles
        private void buttonLoadTiles_Click(object sender, EventArgs e)
        {
            TileWidth = Convert.ToInt32( textTileWidth.Text);
            TileHeight = Convert.ToInt32(textTileHeight.Text);
            TilesAcross = Convert.ToInt32(textTilesAcross.Text);
            TilesDown = Convert.ToInt32(textTilesDown.Text);
            TileBorder = Convert.ToInt32(textTileBorder.Text);

            //grabs the tileset image from the textbox, which was generated from the open file dialog
            string ImageFileName = textImageFileName.Text;

            //creating the tileset bitmap, this may file if the file is invalid or not found, so we look
            //for exceptions
            try
            {
                bmpTileSet = new Bitmap(Image.FromFile(ImageFileName));
            }
            catch (FileNotFoundException)
            {
                MessageBox.Show("File Not Found, Try Again");
                return;
            }
            catch (OutOfMemoryException)
            {
                MessageBox.Show("Not a Valid Image File");
                return;
            }
            
            //set the tileset image to the file and resize
            pictureTileSet.Image = bmpTileSet;
            pictureTileSet.Size = bmpTileSet.Size;

            //creates an array of bitmaps with a size equal to the number of tiles in the file
            bmpTile = new Bitmap[TilesAcross * TilesDown];

            for (int y = 0; y < TilesDown; y++)
            {
                for (int x = 0; x < TilesAcross; x++)
                {
                    int index = x + (y * TilesAcross);

                    bmpTile[index] = new Bitmap(TileWidth, TileHeight);
                    graphics = Graphics.FromImage(bmpTile[index]);

                    //copies the tile from the tileset into the tile array
                    graphics.DrawImage(bmpTileSet,
                        new Rectangle(0, 0, TileWidth, TileHeight),//destination
                        //source rectangle, split to make it easier to follow
                        new Rectangle( (TileBorder*x)+1+(TileWidth*x), //source x
                            (TileBorder*y)+1+(TileHeight*y), //source y
                            TileWidth, //source width
                            TileHeight), //source height
                        GraphicsUnit.Pixel);

                    #region Ugly hack of death (manually sets Tiles to picture boxes)
                    /*begin the ugly hack of death*/
                    //mental note, find a way to iterate through controls
                    if (index == 0)
                    {
                        pictureTile1.Image = bmpTile[index];
                        pictureTile1.Width = TileWidth;
                        pictureTile1.Height = TileHeight;
                    }
                    else if (index == 1)
                    {
                        pictureTile2.Image = bmpTile[index];
                        pictureTile2.Width = TileWidth;
                        pictureTile2.Height = TileHeight;
                    }
                    else if (index == 2)
                    {
                        pictureTile3.Image = bmpTile[index];
                        pictureTile3.Width = TileWidth;
                        pictureTile3.Height = TileHeight;
                    }
                    else if (index == 3)
                    {
                        pictureTile4.Image = bmpTile[index];
                        pictureTile4.Width = TileWidth;
                        pictureTile4.Height = TileHeight;
                    }
                    else if (index == 4)
                    {
                        pictureTile5.Image = bmpTile[index];
                        pictureTile5.Width = TileWidth;
                        pictureTile5.Height = TileHeight;
                    }
                    else if (index == 5)
                    {
                        pictureTile6.Image = bmpTile[index];
                        pictureTile6.Width = TileWidth;
                        pictureTile6.Height = TileHeight;
                    }
                    else if (index == 6)
                    {
                        pictureTile7.Image = bmpTile[index];
                        pictureTile7.Width = TileWidth;
                        pictureTile7.Height = TileHeight;
                    }
                    else if (index == 7)
                    {
                        pictureTile8.Image = bmpTile[index];
                        pictureTile8.Width = TileWidth;
                        pictureTile8.Height = TileHeight;
                    }
                    else if (index == 8)
                    {
                        pictureTile9.Image = bmpTile[index];
                        pictureTile9.Width = TileWidth;
                        pictureTile9.Height = TileHeight;
                    }
                    else if (index == 9)
                    {
                        pictureTile10.Image = bmpTile[index];
                        pictureTile10.Width = TileWidth;
                        pictureTile10.Height = TileHeight;
                    }
                    else if (index == 10)
                    {
                        pictureTile11.Image = bmpTile[index];
                        pictureTile11.Width = TileWidth;
                        pictureTile11.Height = TileHeight;
                    }
                    else if (index == 11)
                    {
                        pictureTile12.Image = bmpTile[index];
                        pictureTile12.Width = TileWidth;
                        pictureTile12.Height = TileHeight;
                    }
                    else
                    {
                        MessageBox.Show("now is the time to learn....");
                    }
                    #endregion

                    TilesLoaded = true;
                    
                }
            }

        }

        //creates the map and loads it with the tiles
        private void buttonLoadMap_Click(object sender, EventArgs e)
        {
            MapTileWidth = Convert.ToInt32(textMapTileWidth.Text);
            MapTileHeight = Convert.ToInt32(textMapTileHeight.Text);

            Map = new int[ MapTileWidth , MapTileHeight ];

            //initialize array to tile ID 0
            for (int x = 0; x < MapTileWidth; x++)
            {
                for (int y = 0; y < MapTileHeight; y++)
                {
                    Map[x, y] = 0;
                }
            }

            //set the selected tile to 0
            SelectedTile = 0;

            MapLoaded = true;
            
        }

        //updates the Map image every tick based on the 2d map array
        private void timerUpdateMap_Tick(object sender, EventArgs e)
        {

            //only update the map if the tiles and map are created and loaded
            if (TilesLoaded && MapLoaded)
            {
                //the map image is sized to the correct dimensions
                Bitmap bmpMap = new Bitmap(MapTileWidth*TileWidth, MapTileHeight*TileHeight);

                for (int x = 0; x < MapTileWidth; x++)
                {
                    for (int y = 0; y < MapTileHeight; y++)
                    {
                        //draw the tile into the correct position on the map image
                        graphics=Graphics.FromImage(bmpMap);
                        graphics.DrawImage(bmpTile[Map[x, y]],
                            new Rectangle(x * TileWidth, y * TileHeight, TileWidth, TileHeight));

                        //set the picture box to the map image and resize
                        pictureMap.Image = bmpMap;
                        pictureMap.Width = MapTileWidth * TileWidth;
                        pictureMap.Height = MapTileHeight * TileHeight;

                    }
                }

                //set the selected tile to the correct image and resize
                pictureSelectedTile.Image = bmpTile[SelectedTile];
                pictureSelectedTile.Size = bmpTile[SelectedTile].Size;

                //if the mouse is down, we want to change the map with the selected tile
                //but only if the mouse is inside the map image
                if (isMouseDown)
                {
                    int mx, my;
                    //where the mouse is in relation to the form
                    Point ClientPosition = PointToClient(new Point(MousePosition.X, MousePosition.Y));

                    //the mouse position is the form position minus the location offset of the panel
                    //minus the location offset of the picture (which includes the scrolling offset)
                    mx = ClientPosition.X - panelMap.Location.X - pictureMap.Location.X;
                    my = ClientPosition.Y - panelMap.Location.Y - pictureMap.Location.Y;

                    //string concat = "mx " + mx + " my " + my;
                    //MessageBox.Show(concat);

                    //What tile the mouse is currently on
                    int TileX,TileY;
                    TileX = mx / TileWidth;
                    TileY = my / TileHeight;

                    //only update the tile if it is inside the image, otherwise we will go out of bounds
                    if (TileX>=0 && TileX < MapTileWidth && TileY>=0 && TileY < MapTileHeight )
                    {
                        Map[TileX, TileY] = SelectedTile;
                    }
                }

            }
        }

        #region Ugly Hack that selects the tile if you click on the Tile picture boxes
        private void pictureTile1_Click(object sender, EventArgs e)
        {
            SelectedTile = 0;
        }

        private void pictureTile2_Click(object sender, EventArgs e)
        {
            SelectedTile = 1;
        }

        private void pictureTile3_Click(object sender, EventArgs e)
        {
            SelectedTile = 2;
        }

        private void pictureTile4_Click(object sender, EventArgs e)
        {
            SelectedTile = 3;
        }

        private void pictureTile5_Click(object sender, EventArgs e)
        {
            SelectedTile = 4;
        }

        private void pictureTile6_Click(object sender, EventArgs e)
        {
            SelectedTile = 5;
        }

        private void pictureTile7_Click(object sender, EventArgs e)
        {
            SelectedTile = 6;
        }

        private void pictureTile8_Click(object sender, EventArgs e)
        {
            SelectedTile = 7;
        }

        private void pictureTile9_Click(object sender, EventArgs e)
        {
            SelectedTile = 8;
        }

        private void pictureTile10_Click(object sender, EventArgs e)
        {
            SelectedTile = 9;
        }

        private void pictureTile11_Click(object sender, EventArgs e)
        {
            SelectedTile = 10;
        }

        private void pictureTile12_Click(object sender, EventArgs e)
        {
            SelectedTile = 11;
        }
        #endregion

        private void pictureMap_MouseDown(object sender, MouseEventArgs e)
        {
            //sets the mousedown bool to true, which will make the painting happen in the timer
            isMouseDown = true;
        }

        private void pictureMap_MouseUp(object sender, MouseEventArgs e)
        {
            isMouseDown = false;
        }

        //copies the map array to a text file so we can copy it over to another file or game
        private void buttonLoadDataGrid_Click(object sender, EventArgs e)
        {
            String DataMapArray="";

            for (int y = 0; y < MapTileHeight; y++)
            {
                DataMapArray += "{";
                for (int x = 0; x < MapTileWidth; x++)
                {
                    DataMapArray += Map[x, y].ToString();
                    //if it's the last one don't add a comma
                    if (x < MapTileWidth - 1)
                    {
                        DataMapArray += ",";
                    }
                }
                DataMapArray += "}";
                if(y < MapTileHeight - 1)
                {
                    DataMapArray+=",\n";
                }
            }

            //update the textbox
            textDataGrid.Text = DataMapArray;
            //copy the text so we can paste it
            textDataGrid.SelectAll();
            textDataGrid.Copy();
        }

        private void pictureMap_Click(object sender, EventArgs e)
        {

        }

        //select the tileset file
        private void buttonChooseTileSet_Click(object sender, EventArgs e)
        {
            //show the open file dialog
            if (openFileDialogLoadTiles.ShowDialog() == DialogResult.OK)
            {
                //update the filename textbox
                textImageFileName.Text = openFileDialogLoadTiles.FileName;
            }
        }


    }
}